--WAF Action
require 'config'
require 'lib'

--args
local rulematch = ngx.re.find
local unescape = ngx.unescape_uri

-- IP白名单检测
function white_ip_check()
    if config_white_ip_check ~= "on" then return false end
    local WHITE_IP = get_client_ip()
    if WHITE_IP == "unknown" then return false end
    local IP_WHITE_RULE = get_rule('whiteip.rule')
    if IP_WHITE_RULE == nil then return false end
    for _,rule in pairs(IP_WHITE_RULE) do
        if rule ~= "" and rulematch(WHITE_IP,rule,"jo") then
            log_record('White_IP',ngx.var_request_uri,"_","_")
            return true
        end
    end
end

-- IP黑名单检测
function black_ip_check()
    if config_black_ip_check ~= "on" then return false end
    local BLACK_IP = get_client_ip()
    if BLACK_IP == "unknown" then return false end
    local IP_BLACK_RULE = get_rule('blackip.rule')
    if IP_BLACK_RULE == nil then return false end
    local BLACK_IP = get_client_ip()
    for _,rule in pairs(IP_BLACK_RULE) do
        if rule ~= "" and rulematch(BLACK_IP,rule,"jo") then
            log_record('BlackList_IP',ngx.var_request_uri,"_","_")
            if config_waf_enable == "on" then
                ngx.exit(403)
                return true
            end
        end
    end
end

-- 用户代理检测
function user_agent_attack_check()
    if config_user_agent_check ~= "on" then return false end
    local USER_AGENT = ngx.var.http_user_agent
    if USER_AGENT == nil then return false end
    local USER_AGENT_RULES = get_rule('useragent.rule')
    if USER_AGENT_RULES == nil then return false end
    for _,rule in pairs(USER_AGENT_RULES) do
        if rule ~="" and rulematch(USER_AGENT,rule,"jo") then
            log_record('Deny_USER_AGENT',ngx.var.request_uri,"-",rule)
            if config_waf_enable == "on" then
                waf_output()
                return true
            end
        end
    end
end

-- CC攻击检测
function cc_attack_check()
    if config_cc_check == "on" then return false end
    local ATTACK_URI = ngx.var.uri
    local CC_TOKEN = get_client_ip()..ATTACK_URI
    local limit = ngx.shared.limit
    CCcount = tonumber(string.match(config_cc_rate,'(.*)/'))
    CCseconds = tonumber(string.match(config_cc_rate,'/(.*)'))
    local req,_ = limit:get(CC_TOKEN)
    if req then
        if req > CCcount then
            log_record('CC_Attack',ngx.var.request_uri,"-","-")
            if config_waf_enable == "on" then
                ngx.exit(403)
            end
        else
            limit:incr(CC_TOKEN,1)
        end
    else
        limit:set(CC_TOKEN,1,CCseconds)
    end
end

-- COOKIE攻击检测
function cookie_attack_check()
    if config_cookie_check ~= "on" then return false end
    local USER_COOKIE = ngx.var.http_cookie
    if USER_COOKIE == nil then return false end
    local COOKIE_RULES = get_rule('cookie.rule')
    if COOKIE_RULES == nil then return false end
    for _,rule in pairs(COOKIE_RULES) do
        if rule ~="" and rulematch(USER_COOKIE,rule,"jo") then
            log_record('Deny_Cookie',ngx.var.request_uri,"-",rule)
            if config_waf_enable == "on" then
                waf_output()
                return true
            end
        end
    end
end

-- URL白名单检测
function white_url_check()
    if config_white_url_check ~= "on" then return false end
    local REQ_URI = ngx.var.request_uri
    if REQ_URI == nil then return false end
    local URL_WHITE_RULES = get_rule('whiteurl.rule')
    if URL_WHITE_RULES == nil then return false end
    for _,rule in pairs(URL_WHITE_RULES) do
        if rule ~= "" and rulematch(REQ_URI,rule,"jo") then
            return true
        end
    end
end

-- URL受限访问攻击检测
function url_attack_check()
    if config_url_check ~= "on" then return false end
    local REQ_URI = ngx.var.request_uri
    if REQ_URI == nil then return false end
    local URL_RULES = get_rule('url.rule')
    if URL_RULES == nil then return false end
    for _,rule in pairs(URL_RULES) do
        if rule ~="" and rulematch(REQ_URI,rule,"jo") then
            log_record('Deny_URL',REQ_URI,"-",rule)
            if config_waf_enable == "on" then
                waf_output()
                return true
            end
        end
    end
end

-- URL参数注入攻击检测
function url_args_attack_check()
    if config_url_args_check ~= "on" then return false end
    local REQ_ARGS = ngx.req.get_uri_args()
    if REQ_ARGS == nil then return false end
    local ARGS_RULES = get_rule('args.rule')
    if ARGS_RULES == nil then return false end
    for key, val in pairs(REQ_ARGS) do
        if type(val) == 'table' then
            ARGS_DATA = table.concat(val, " ")
        else
            ARGS_DATA = val
        end
        if ARGS_DATA and type(ARGS_DATA) ~= "boolean" then
            for _,rule in pairs(ARGS_RULES) do
                if rule ~="" and rulematch(unescape(ARGS_DATA),rule,"jo") then
                    log_record('Deny_URL_Args',ngx.var.request_uri,"-",rule)
                    if config_waf_enable == "on" then
                        waf_output()
                        return true
                    end
                end
            end
        end
    end
end

-- POST参数注入攻击检测
function post_attack_check()
    if config_post_check ~= "on" then return false end
	if get_content_type() ~= "application/x-www-form-urlencoded" then return false end
    ngx.req.read_body()
    local POST_ARGS = ngx.req.get_post_args()
    if POST_ARGS == nil then return false end
    local POST_RULES = get_rule('post.rule')
    if POST_RULES == nil then return false end
    for key, val in pairs(POST_ARGS) do
        if type(val) == 'table' then
            POST_DATA = table.concat(val, " ")
        else
            POST_DATA = val
        end
        if POST_DATA and type(POST_DATA) ~= "boolean" then
            for _,rule in pairs(POST_RULES) do
                if rule ~="" and rulematch(unescape(POST_DATA),rule,"jo") then
                    log_record('Deny_POST_Args',ngx.var.request_uri,"-",rule)
                    if config_waf_enable == "on" then
                        waf_output()
                        return true
                    end
                end
            end
        end
    end
end